#pragma once

#include <stdint.h>
#include <LSM6.h>
#include <Balboa32U4.h>


// This code was developed for a Balboa unit using 75:1 motors
// and 49:17 plastic gears, for an overall gear ratio of 220.
const int16_t GEAR_RATIO = 220;
const int16_t MOTOR_SPEED_LIMIT = 300;
// 360 degree / Pi x D  
// (360 / (PI x 80)) x 100 = 143.2
// (90 / 143.2) x 1000 = 628 deg / s
const int16_t ANGLE_RATE_RATIO = 143;
const int16_t ANGLE_RESPONSE = 28;
const int16_t DISTANCE_RESPONSE = 37;
const int16_t DISTANCE_DIFF_RESPONSE = -50;
const int16_t SPEED_RESPONSE = 3300;
const uint8_t UPDATE_TIME_MS = 10;
const uint8_t CALIBRATION_ITERATIONS = 100;
const int32_t START_BALANCING_ANGLE = 45000;
const int32_t STOP_BALANCING_ANGLE = 70000;
// total number of line sensors.
const uint8_t SensorCount = 6;

extern LSM6 imu;
extern Balboa32U4Motors motors;
extern Balboa32U4Encoders encoders;
extern QTRSensors lineSensors;

extern int16_t motorSpeed;
extern int32_t speed_l, speed_r;
extern int32_t distance_l, distance_r;
extern int32_t angle, angleRate;
extern uint16_t sensorValues [SensorCount];
//extern int32_t turn, turnRate;

void balanceSetup ();
void balanceUpdate ();
void balanceDrive (int16_t leftSpeed, int16_t rightSpeed);
bool isBalancing ();
bool balanceUpdateDelayed ();
void balanceUpdateSensors ();
void balanceResetEncoders ();
