#include <Wire.h>
#include "Balance.h"

// Global variables
int16_t motorSpeed;
int32_t distance_l, distance_r;
int32_t speed_l, speed_r;
int32_t driveLeft, driveRight;
int32_t gYZero, angle, angleRate;
//int32_t gXZero, turn, turnRate;
uint16_t sensorValues [SensorCount];

bool isBalancingStatus = false;
bool balanceUpdateDelayedStatus;

bool isBalancing () {
  return isBalancingStatus;
}

bool balanceUpdateDelayed () {
  return balanceUpdateDelayedStatus;
}

void balanceSetup () {
  Wire.begin ();

  // Configure the line sensor library.
  // lineSensor.setCenterAligned();
  // lineSensors.setEdgeAligned ();
  lineSensors.setTypeRC ();
  lineSensors.setSensorPins ((const uint8_t[]){5, A4, A3, A2, A0, 12}, SensorCount);
//  lineSensors.setSensorPins ((const uint8_t[]){12, A0, A2, A3, A4, 5}, SensorCount);
  lineSensors.setEmitterPin (QTRNoEmitterPin);
  
  imu.init ();
  imu.enableDefault ();
  imu.writeReg (LSM6::CTRL2_G, 0b01011000);
  delay (1000);

  int32_t total = 0;
//  int32_t total_x = 0;
  for (int i = 0; i < CALIBRATION_ITERATIONS; i++) {
    imu.read ();
    total += imu.g.y;
//    total_x += imu.g.x;
    delay (1);
  }
  gYZero = total / CALIBRATION_ITERATIONS;
//  gXZero = total_x / CALIBRATION_ITERATIONS;
}

void balance () {
  
  angle = angle * 999 / 1000;
  int32_t risingAngleOffset = angleRate * ANGLE_RATE_RATIO  + angle;

  motorSpeed += (
    + ANGLE_RESPONSE * risingAngleOffset
    + DISTANCE_RESPONSE * (distance_l + distance_r)
    + SPEED_RESPONSE * (speed_l + speed_r)
    ) / 100 / GEAR_RATIO;

  motorSpeed = constrain (motorSpeed, -MOTOR_SPEED_LIMIT, MOTOR_SPEED_LIMIT);
  int16_t distanceDiff = distance_l - distance_r;
  motors.setSpeeds(
    motorSpeed + distanceDiff * DISTANCE_DIFF_RESPONSE / 100,
    motorSpeed - distanceDiff * DISTANCE_DIFF_RESPONSE / 100);
}

void lyingDown () {
  motorSpeed = 0;
  distance_l = 0;
  distance_r = 0;
  motors.setSpeeds (0, 0);
  
  if (angleRate > -2 && angleRate < 2) {
    // It's really calm, so use the accelerometer to measure the
    // robot's rest angle.  The atan2 function returns a result
    // in radians, so we multiply it by 180000/pi to convert it
    // to millidegrees.
    angle = atan2(imu.a.z, imu.a.x) * 57296;

    distance_l = 0;
    distance_r = 0;
  }
}

void integrateGyro () {
  angleRate = (imu.g.y - gYZero) / 29;
//  turnRate = (imu.g.x - gXZero) / 29;
  angle += angleRate * UPDATE_TIME_MS;
//  turn += turnRate * UPDATE_TIME_MS;
}

void integrateEncoders () {
  static int16_t lastCountsLeft, lastCountsRight;
  int16_t countsLeft, countsRight;
  
  countsLeft = encoders.getCountsLeft ();
  countsRight = encoders.getCountsRight ();
  speed_l = (countsLeft - lastCountsLeft);
  distance_l += countsLeft - lastCountsLeft;
  lastCountsLeft = countsLeft;
  speed_r = (countsRight - lastCountsRight);
  distance_r += countsRight - lastCountsRight;
  lastCountsRight = countsRight;
}

void balanceDrive (int leftSpeed, int rightSpeed) {
  driveLeft = leftSpeed;
  driveRight = rightSpeed;
}

void balanceDoDriveTicks () {
  distance_l -= driveLeft;
  speed_l -= driveLeft;
  distance_r -= driveRight;
  speed_r -= driveRight;
}

void balanceResetEncoders () {
  distance_l = distance_r = 0;
}

void balanceUpdateSensors () {
  imu.read ();
  integrateGyro ();
  integrateEncoders ();
  // read the line sensor.
  lineSensors.read (sensorValues);
}

void balanceUpdate () {
  static uint16_t lastMillis;
  uint16_t ms = millis ();
  static uint8_t count = 0;
  
  if ((uint16_t)(ms - lastMillis) < UPDATE_TIME_MS) {
    return;
  }
  balanceUpdateDelayedStatus = ms - lastMillis > UPDATE_TIME_MS + 1;
  lastMillis = ms;
  
  balanceUpdateSensors ();
  balanceDoDriveTicks ();

  if (isBalancingStatus)
  {
    balance();

    // Stop trying to balance if we have been farther from
    // vertical than STOP_BALANCING_ANGLE for 5 counts.
    if (abs(angle) > STOP_BALANCING_ANGLE)
    {
      if (++count > 5)
      {
        isBalancingStatus = false;
        count = 0;
      }
    }
    else
    {
      count = 0;
    }
  }
  else
  {
    lyingDown();

    // Start trying to balance if we have been closer to
    // vertical than START_BALANCING_ANGLE for 5 counts.
    if (abs(angle) < START_BALANCING_ANGLE)
    {
      if (++count > 5)
      {
        isBalancingStatus = true;
        count = 0;
      }
    }
    else
    {
      count = 0;
    }
  }
}
